@@ -26,6 +26,7 @@
 #include <QtQml/QtQml>
 #include <QLibrary>
 #include <QDir>
+#include <QScreen>
 
 #include <QDebug>
 
@@ -73,6 +74,14 @@ int main(int argc, char *argv[])
     } else {
         view.engine()->rootContext()->setContextProperty("noAuthentication", QVariant(false));
     }
+
+    QScreen* screen = QGuiApplication::primaryScreen();
+    QSizeF size = screen->physicalSize();
+
+    view.engine()->rootContext()->setContextProperty("physicalDotsPerInch", screen->physicalDotsPerInch());
+    view.engine()->rootContext()->setContextProperty("screenPhysicalWidth", size.width());
+    view.engine()->rootContext()->setContextProperty("screenPhysicalHeight", size.height());
+
     for (int i = 0; i < args.count(); i++) {
         if (args.at(i) == "-I" && args.count() > i + 1) {
             QString addedPath = args.at(i+1);
