@@ -480,9 +480,8 @@ void DirModel::setPathWithAuthentication(const QString &path, const QString &use
     setPath(path, user, password, savePassword);
 }
 
-
 void DirModel::setPath(const QString &pathName, const QString &user, const QString &password,
-                       bool savePassword)
+                       bool savePassword, bool clearForwardHistory)
 {
     if (pathName.isEmpty())
         return;
@@ -516,7 +515,7 @@ void DirModel::setPath(const QString &pathName, const QString &user, const QStri
     }
 
     mCurLocation = location;
-    setPathFromCurrentLocation();
+    setPathFromCurrentLocation(clearForwardHistory);
 }
 
 /*!
@@ -524,7 +523,7 @@ void DirModel::setPath(const QString &pathName, const QString &user, const QStri
  *
  *  Used in \ref cdUp() and \ref cdIntoIndex()
  */
-void DirModel::setPathFromCurrentLocation()
+void DirModel::setPathFromCurrentLocation(bool clearForwardHistory)
 {
     mAwaitingResults = true;
     emit awaitingResultsChanged();
@@ -550,7 +549,14 @@ void DirModel::setPathFromCurrentLocation()
     else
         mCurLocation->fetchItems(currentDirFilter(), mIsRecursive);
 
+    if (clearForwardHistory) {
+        mPathForwardList.clear();
+        emit canGoForwardChanged();
+    }
+
     emit canGoBackChanged();
+    emit canGoHomeChanged();
+    emit canGoUpChanged();
     emit pathChanged(mCurLocation->urlPath());
 }
 
@@ -562,12 +568,32 @@ bool DirModel::canGoBack() const
 void DirModel::goBack()
 {
     if (mPathList.count() > 1 && !mAwaitingResults) {
+        mPathForwardList.append(mPathList.last());
         mPathList.removeLast();
 
 #if DEBUG_MESSAGES
         qDebug() << Q_FUNC_INFO << this << "changing to" << mPathList.last();
 #endif
-        setPath(mPathList.last());
+        setPath(mPathList.last(), "", "", false, false);
+        emit canGoForwardChanged();
+    }
+}
+
+bool DirModel::canGoForward() const
+{
+    return mPathForwardList.count() > 0;
+}
+
+void DirModel::goForward()
+{
+    if (mPathForwardList.count() > 0 && !mAwaitingResults) {
+        setPath(mPathForwardList.last(), "", "", false, false);
+        mPathForwardList.removeLast();
+
+#if DEBUG_MESSAGES
+        qDebug() << Q_FUNC_INFO << this << "changing to" << mPathForwardList.last();
+#endif
+        emit canGoForwardChanged();
     }
 }
 
@@ -936,6 +962,11 @@ ExternalFSWatcher *DirModel::getExternalFSWatcher() const
 }
 #endif
 
+bool DirModel::canGoHome() const
+{
+    return path() != homePath();
+}
+
 void DirModel::goHome()
 {
     setPath(QDir::homePath());
@@ -946,6 +977,11 @@ void DirModel::goTrash()
     setPath(LocationUrl::TrashRootURL);
 }
 
+bool DirModel::canGoUp() const
+{
+    return path() != "/";
+}
+
 bool DirModel::cdUp()
 {
     int ret = mCurLocation && mCurLocation->becomeParent();
