@@ -58,6 +58,9 @@ class DirModel : public DirItemAbstractListModel, public QQmlParserStatus
     Q_PROPERTY(int count READ rowCount NOTIFY countChanged)
     Q_PROPERTY(QString path READ path WRITE setPath NOTIFY pathChanged)
     Q_PROPERTY(bool canGoBack READ canGoBack NOTIFY canGoBackChanged)
+    Q_PROPERTY(bool canGoForward READ canGoForward NOTIFY canGoForwardChanged)
+    Q_PROPERTY(bool canGoHome READ canGoHome NOTIFY canGoHomeChanged)
+    Q_PROPERTY(bool canGoUp READ canGoUp NOTIFY canGoUpChanged)
     Q_PROPERTY(bool awaitingResults READ awaitingResults NOTIFY awaitingResultsChanged)
     Q_PROPERTY(bool filterDirectories READ filterDirectories WRITE setFilterDirectories NOTIFY filterDirectoriesChanged)
     Q_PROPERTY(bool isRecursive READ isRecursive WRITE setIsRecursive NOTIFY isRecursiveChanged)
@@ -262,7 +265,7 @@ public:
     Q_INVOKABLE void refresh()
     {
         // just some syntactical sugar really
-        setPath(path());
+        setPath(path(), "", "", false, false);
     }
 
     static void registerMetaTypes();
@@ -286,9 +289,12 @@ public:
         return mCurrentDir;
     }
     void setPath(const QString &pathName, const QString &user = QString(),
-                 const QString &password = QString(), bool savePassword = false);
+                 const QString &password = QString(), bool savePassword = false, bool clearForwardHistory = true);
 
     bool canGoBack() const;
+    bool canGoForward() const;
+    bool canGoHome() const;
+    bool canGoUp() const;
     bool awaitingResults() const;
     bool filterDirectories() const;
     bool isRecursive() const;
@@ -394,6 +400,12 @@ public slots:   // Also invokable from QML
      */
     void goBack();
 
+    /*!
+     * \brief goForward() goes to the next location if available
+     *
+     */
+    void goForward();
+
     /*!
      * \brief cdUp() sets the parent directory as current directory
      *
@@ -489,6 +501,9 @@ private:
 signals:
     void countChanged();
     void canGoBackChanged();
+    void canGoForwardChanged();
+    void canGoHomeChanged();
+    void canGoUpChanged();
     void awaitingResultsChanged();
     void nameFiltersChanged();
     void filterDirectoriesChanged();
@@ -563,7 +578,7 @@ private:
     bool          cdIntoItem(const DirItemInfo &fi);
     bool          openItem(const DirItemInfo &fi);
     DirItemInfo   setParentIfRelative(const QString &fileOrDir) const;
-    void          setPathFromCurrentLocation();
+    void          setPathFromCurrentLocation(bool clearForwardHistory = true);
 
 private:
     void          startExternalFsWatcher();
@@ -592,6 +607,7 @@ private:
     LocationsFactory   *mLocationFactory;
     Location           *mCurLocation;
     QStringList         mPathList;    //!< it will be used for goBack()
+    QStringList         mPathForwardList;    //!< it will be used for goForward()
     QString             mQmlCachePath;
 
 private:
